const fs = require('fs');
const path = require('path');

const { createFilePath } = require('gatsby-source-filesystem');

exports.onPreBootstrap = ({ reporter }) => {
  const apiSpecsPath = 'src/api-specs';
  const dataPath = 'src/data';
  const pagesPath = 'src/pages';
  const imagesPath = 'src/images';
  const filesPath = 'src/pages/files';

  if (!fs.existsSync(apiSpecsPath)) {
    reporter.info(`creating the ${apiSpecsPath} directory`);
    fs.mkdirSync(apiSpecsPath);
  }

  if (!fs.existsSync(dataPath)) {
    reporter.info(`creating the ${dataPath} directory`);
    fs.mkdirSync(dataPath);
  }

  if (!fs.existsSync(pagesPath)) {
    reporter.info(`creating the ${pagesPath} directory`);
    fs.mkdirSync(pagesPath);
  }

  if (!fs.existsSync(imagesPath)) {
    reporter.info(`creating the ${imagesPath} directory`);
    fs.mkdirSync(imagesPath);
  }

  if (!fs.existsSync(filesPath)) {
    reporter.info(`creating the ${filesPath} directory`);
    fs.mkdirSync(filesPath);
  }
};

exports.sourceNodes = ({ actions }) => {
  actions.createTypes(`
    type Site implements Node {
      siteMetadata: SiteMetadata!
    }

    type SiteMetadata {
      siteUrl: String
      title: String!
      shortTitle: String
      description: String!
      author: String
    }

    type NavigationYaml implements Node @dontInfer {
      id: ID!
      sectionTitle: String! @proxy(from: "section-title")
      entries: [Entry!]
    }

    type Entry {
      title: String!
      path: String!
      beta: Boolean
    }
  `);
};

exports.onCreateNode = ({ node, getNode, actions }) => {
  if (node.internal.type === 'Mdx') {
    const { createNodeField } = actions;
    const slug = createFilePath({ node, getNode, basePath: 'pages' });

    createNodeField({
      node,
      name: 'slug',
      value: slug
    });
  }
};

exports.onCreatePage = ({ page, actions }) => {
  const extensionName = path.extname(page.componentPath);

  if (extensionName === '.md' || extensionName === '.mdx') {
    createPageForMdx(page, actions);
  }

  function createPageForMdx(page, actions) {
    const { createPage, deletePage } = actions;

    deletePage(page);

    createPage({
      ...page,
      // page.context.frontmatter can be used to dynamically select approriate templates
      component: require.resolve('./src/templates/mdx.js'),
      context: {
        ...page.context,
        // Data passed to context is available
        // in page queries as GraphQL variables.
        slug: page.path
      }
    });
  }
};

// this is necessary for importing own local components in MDX
// because gatsby-MDX does not support the concept of a relative path
/* Wale's Note:
Adding Webpack resolve settings to be able to include React components in mdx files
and without relative paths means we lose access to hot reloading when included files are changed.
*/
exports.onCreateWebpackConfig = ({ stage, plugins, actions }) => {
  actions.setWebpackConfig({
    plugins: [
      plugins.define({
        __DEVELOPMENT__: stage === 'develop' || stage === 'develop-html'
      })
    ],
    resolve: {
      modules: [require.resolve(__dirname, 'src/components'), 'node_modules']
    }
  });
};
