import React from 'react';
import PropTypes from 'prop-types';
import { Location } from '@reach/router';
import { Link as GatsbyLink } from 'gatsby';
import { OutboundLink } from 'gatsby-plugin-google-analytics';
import { URL as nodeURL } from 'url';
import { useSiteMetadata } from '../../utils/use-site-metadata';

// isomorphic URL parser:
const URL = nodeURL ? nodeURL : window.URL;

/**
 * This Link wrapper resolves any sort of relative URL addressing to a URL object and
 * uses that to generate paths starting with / because Gatsby's Link component understands
 * only these.
 *
 * In addition it generates Google Analytics tracked outbound links and plain anchors for
 * on-page links because Gatsby Link does not support these either.
 *
 * As a commercetools-specific addition, you can force a non-gatsby link with the
 * following convention: all links are interpreted as-is without a path prefix except ones
 * starting with "/../*" which are considered pointing to another
 * microsite as "/microsite/pathOnMicrosite". These are rendered as plain <a> tags.
 * */

export const PureLink = ({
  location,
  configuredPrefix,
  isDevelopment,
  href = '',
  children,
  ...otherProps
}) => {
  // reach router location includes the current Prefix, so for the purpose of resolving relative paths
  // not starting with a slash to as if they were written starting with a slash (only these are compatible with gatsby)
  // we remove the prefix from the location handed by reach router:
  const currentPathname = location.pathname.replace(
    new RegExp('^' + configuredPrefix),
    ''
  );
  const currentHash = location.hash;

  // since some overly eager plugins (mdx?) and components add the pathPrefix by themselves although
  // Gatsby is in fact adding it automatically, we purge such cases from the beginning of /foo style hrefs
  const normalizedHref = href.replace(new RegExp('^' + configuredPrefix), '');

  // use a real URL parser to resolve all sorts of relative addressing correctly:
  const hrefAsUrl = new URL(
    normalizedHref,
    'https://dummyhost' + currentPathname + currentHash
  );

  // case 1: Outbound link on other domain, no further logic required
  if (hrefAsUrl.host != 'dummyhost') {
    return (
      <OutboundLink href={href} data-info="outbound-link" {...otherProps}>
        {children}
      </OutboundLink>
    );
  }

  // Case 2: Points to same microsite (=gatsbyJS app) but not to same page = use GatsbyLink magic
  const pointsToDifferentMicrosite = normalizedHref.startsWith('/../');
  const pointsToSamePage = hrefAsUrl.pathname === currentPathname;
  if (!pointsToDifferentMicrosite && !pointsToSamePage) {
    return (
      <GatsbyLink
        to={hrefAsUrl.pathname + hrefAsUrl.hash}
        {...otherProps}
        data-info="gatsby-link"
      >
        {children}
      </GatsbyLink>
    );
  }

  // Case 3 development mode runs only a part of the overall site, tell otherwise confused users that.
  if (isDevelopment && pointsToDifferentMicrosite) {
    return (
      <a
        {...otherProps}
        data-info="html-link"
        onClick={() => {
          alert(
            'Link to other microsite.\n  Only available in deployed preview builds. \nURL: ' +
              normalizedHref
          );
          return false;
        }}
      >
        {children}
      </a>
    );
  }

  // Case 4 when effectively pointing to same page use only hash to avoid gatsby adding path prefixes
  if (pointsToSamePage) {
    return (
      <a href={hrefAsUrl.hash} {...otherProps} data-info="html-link">
        {children}
      </a>
    );
  }

  // Case 5 any other link becomes a plain anchor:
  return (
    <a href={normalizedHref} {...otherProps} data-info="html-link">
      {children}
    </a>
  );
};

PureLink.propTypes = {
  location: PropTypes.object.isRequired,
  configuredPrefix: PropTypes.string.isRequired,
  isDevelopment: PropTypes.bool.isRequired,
  href: PropTypes.string.isRequired,
  children: PropTypes.node
};

const Link = ({ href, children, ...otherProps }) => {
  const configuredPrefix = useSiteMetadata().pathPrefix;
  const isDevelopment = process.env.NODE_ENV === 'development';
  return (
    <Location>
      {({ location }) => {
        return (
          <PureLink
            location={location}
            configuredPrefix={configuredPrefix}
            isDevelopment={isDevelopment}
            href={href}
            {...otherProps}
          >
            {children}
          </PureLink>
        );
      }}
    </Location>
  );
};

Link.propTypes = {
  href: PropTypes.string.isRequired,
  children: PropTypes.node
};

export default Link;
