import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { StaticQuery, graphql, Link } from 'gatsby';
import styled from '@emotion/styled';

import { useSiteMetadata } from '../../utils/use-site-metadata';
import { computePathWithPathPrefix } from '../../utils/helpers';
import { navPaddingRight } from '../styles/dimensions';
import LinesClamper from '../lines-clamper';

const NavElement = styled.nav`
  height: 100%;
  width: 100%;
  padding-right: ${navPaddingRight}rem;
`;

const SectionsUl = styled.ul`
  overflow-y: scroll;
`;

const SectionLi = styled.li`
  padding: 0.5rem 0;

  :not(:first-of-type) {
    border-top: 1px solid #e6e6e6;
  }

  :first-of-type {
    padding: 0.75rem 0;

    p {
      font-size: 1.25rem;
      line-height: 1.625rem;
      text-transform: uppercase;
    }
  }
`;

const SectionLiP = styled.p`
  color: #1a1a1a;
  font-size: 1rem;
  line-height: 1.438rem;
  padding: 0.25rem 1rem;
  word-break: break-word;
`;

const EntryLi = styled.li`
  font-size: 0.875rem;
  line-height: 1.25rem;
  align-items: center;
`;

const EntryLiDiv = styled.div`
  display: flex;

  div.selected {
    padding-left: 0.8125rem;
  }
`;

const ActiveStateBlueBar = styled.div`
  width: 0.1875rem;
  border-radius: 0 0.09375rem 0.09375rem 0;
  background-color: #078cdf;

  :before {
    content: '';
    display: block;
    padding-top: 100%;
  }
`;

const EntryLiDivTextContainer = styled.div`
  flex: 1;
  display: flex;
  padding: 0.25rem 0 0.25rem 1rem;

  /* remove when link custom styling is implemented */
  .selected {
    a {
      color: #078cdf;
    }
  }
`;

const EntryLiPSpan = styled.p`
  a {
    color: #666;

    :hover {
      color: #078cdf;
      text-decoration: none;
    }
  }
`;

const BetaContainer = styled.div`
  display: inline-flex;
  flex-direction: column;
  justify-content: center;
  margin-left: 0.25rem;
`;

const Beta = styled.span`
  color: #078cdf;
  font-size: 0.625rem;
  line-height: 0.8125rem;
  border: 0.0625rem solid #078cdf;
  border-radius: 0.25rem;
  padding: 0.0625rem 0.25rem;
`;

export const PureLeftNav = ({ title, navItems, selected }) => {
  return (
    <NavElement>
      <SectionsUl>
        <SectionLi>
          <SectionLiP>{title}</SectionLiP>
        </SectionLi>
        {navItems.map(navItem => (
          <SectionLi key={navItem.sectionTitle}>
            <SectionLiP>
              <LinesClamper>{navItem.sectionTitle}</LinesClamper>
            </SectionLiP>
            {navItem.entries ? (
              <ul>
                {navItem.entries.map(entry => {
                  const path = computePathWithPathPrefix(entry.path);
                  const thisIsSelected = path === selected;

                  return (
                    <EntryLi key={entry.title}>
                      <EntryLiDiv>
                        {thisIsSelected ? <ActiveStateBlueBar /> : null}

                        <EntryLiDivTextContainer
                          className={thisIsSelected ? 'selected' : null}
                        >
                          <EntryLiPSpan
                            className={thisIsSelected ? 'selected' : null}
                          >
                            <Link data-tip="hello world" to={entry.path}>
                              <LinesClamper>{entry.title}</LinesClamper>
                            </Link>
                          </EntryLiPSpan>

                          {entry.beta ? (
                            <BetaContainer>
                              <Beta>BETA</Beta>
                            </BetaContainer>
                          ) : null}
                        </EntryLiDivTextContainer>
                      </EntryLiDiv>
                    </EntryLi>
                  );
                })}
              </ul>
            ) : null}
          </SectionLi>
        ))}
      </SectionsUl>
    </NavElement>
  );
};

PureLeftNav.propTypes = {
  title: PropTypes.string.isRequired,
  navItems: PropTypes.arrayOf(
    PropTypes.shape({
      sectionTitle: PropTypes.string.isRequired,
      entries: PropTypes.arrayOf(
        PropTypes.shape({
          title: PropTypes.string.isRequired,
          path: PropTypes.string.isRequired,
          beta: PropTypes.bool
        })
      )
    })
  ).isRequired,
  selected: PropTypes.string
};

const LeftNav = props => {
  const { siteMetadata } = useSiteMetadata();
  const title = siteMetadata.shortTitle || siteMetadata.title;

  return (
    <StaticQuery
      query={graphql`
        query NavigationQuery {
          allNavigationYaml {
            nodes {
              sectionTitle
              entries {
                title
                path
                beta
              }
            }
          }
        }
      `}
      render={data => (
        <PureLeftNav
          title={title}
          navItems={data.allNavigationYaml.nodes}
          {...props}
        />
      )}
    />
  );
};

const mapStateToProps = state => ({
  selected: state.leftNav.selected
});

export default connect(mapStateToProps)(LeftNav);
