import React, { useEffect } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import styled from '@emotion/styled';

import { navPaddingRight } from '../../styles/dimensions/left';
import { sectionLeftPadding } from '../../styles/dimensions/right';
import { setSelected } from '../../../action-creators/right-nav';
import RightNavTitleIcon from './right-nav-title-icon';
import NavItem from './nav-item';

const Nav = styled.nav`
  width: 100%;
  padding-right: ${navPaddingRight}rem;
  position: sticky;
  top: 0;
`;

const TitleContainer = styled.div`
  display: flex;
  align-items: center;
  padding-left: ${sectionLeftPadding}rem;
`;

const TitleAnchor = styled.a`
  color: #666;

  :hover {
    text-decoration: none;
  }
`;

const Title = styled.p`
  color: #666;
  font-size: 0.875rem;
  line-height: 1.25rem;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
`;

const Ul = styled.ul`
  margin-top: 1rem;

  li.section:not(:first-of-type) {
    margin-top: 0.5rem;
  }
`;

function Right({ title, items, setSelected, selected, dataTestId }) {
  let sections = initSectionsKey(items);
  let isScrolling;

  useEffect(() => {
    if (!dataTestId) {
      sections = getAnchorPoints(sections);
      window.addEventListener('scroll', handleScroll);
    }
  });

  return (
    <Nav>
      <TitleContainer>
        <TitleAnchor href={`#${title.id}`}>
          <Title>{title.text}</Title>
        </TitleAnchor>
        <RightNavTitleIcon />
      </TitleContainer>

      <Ul>
        {items.map(item => (
          <NavItem
            key={item.url}
            item={item}
            onNavItemClick={selectMenuItem}
            selected={selected === item.title ? true : false}
            className="section"
          />
        ))}
      </Ul>
    </Nav>
  );

  function initSectionsKey(items) {
    const sections = [];

    items.forEach(item => {
      const section = { ...item, id: item.url.substring(1) };
      sections.push(section);
    });

    return sections;
  }

  function getAnchorPoints(sections) {
    const sectionsWithAnchorPointsValues = [];

    const curScroll = window.scrollY;

    for (let section of sections) {
      sectionsWithAnchorPointsValues.push({
        ...section,
        anchorPoint: Math.floor(
          // Can use document.querySelector so can use url directly
          document.getElementById(section.id).getBoundingClientRect().top +
            curScroll
        )
      });
    }

    return sectionsWithAnchorPointsValues;
  }

  function handleScroll() {
    // Clear our timeout throughout the scroll
    window.clearTimeout(isScrolling);

    // Set a timeout to run after scrolling ends
    isScrolling = setTimeout(function() {
      // Run the callback
      const curPos = window.scrollY;

      const currentSection = computeCurrentSection(curPos, sections);

      if (currentSection) setSelected(currentSection.title);
      else setSelected('');
    }, 20);
  }

  function computeCurrentSection(curPos, sections) {
    let currentSection;

    for (let index = sections.length - 1; index >= 0; index--) {
      if (curPos >= sections[index].anchorPoint) {
        currentSection = sections[index];
        break;
      }
    }

    return currentSection;
  }

  function selectMenuItem(itemName) {
    if (setSelected) {
      setSelected(itemName);
    }
  }
}

Right.propTypes = {
  title: PropTypes.shape({
    text: PropTypes.string,
    id: PropTypes.string
  }),
  items: PropTypes.array.isRequired,
  setSelected: PropTypes.func,
  selected: PropTypes.string,
  dataTestId: PropTypes.string
};

const mapStateToProps = state => ({
  selected: state.rightNav.selected
});

const mapDispatchToProps = { setSelected };

export default connect(
  mapStateToProps,
  mapDispatchToProps
)(Right);
