import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import styled from '@emotion/styled';
import { css } from '@emotion/core';

import { toggleOpenLeftNav } from '../../action-creators/navigation-drawer';
import {
  headerHeight,
  headerBoxShadowOffsetY,
  headerBoxShadowBlurRadius2Pixel,
  headerBoxShadowBlurRadius3Pixel,
  navigationAndMainContentTop,
  navContainerWidth,
  navContainerBorderRight,
  scrollBarWidth
} from '../styles/dimensions';
import LeftNavButton from './left-nav-button';

const Container = styled.div`
  width: 100%;
  height: 100%;

  #content {
    width: 100%;
    height: 100%;
  }

  #content::after {
    position: fixed;
    top: 0;
    right: 0;
    content: '';
    background-color: rgba(0, 0, 0, 0.4);
    overflow: hidden;
    opacity: 0;
    transition: opacity 0.5s;
  }

  #content.show-transparent::after {
    width: 100%;
    height: 100%;
    opacity: 1;
  }

  .show-navigation {
    left: 0;
    box-shadow: 0 14px 28px 0 rgba(0, 0, 0, 0.25),
      0 10px 10px 0 rgba(0, 0, 0, 0.22);
  }

  @media (max-width: 68.63rem) {
    #left-nav {
      display: none;
    }

    #navigation-button-for-mobile {
      display: inline-block;
    }

    #header-container {
      margin-left: 2rem;
    }
  }
`;

const LeftNavButtonContainer = styled.div`
  display: none;
`;

const NavButtonHeaderContainer = styled.div`
  position: sticky;
  top: 0;
  display: flex;
  height: ${headerHeight}rem;
  background-color: #fff;
  box-shadow: 0 ${headerBoxShadowOffsetY}rem
      ${headerBoxShadowBlurRadius2Pixel}rem 0 rgba(0, 0, 0, 0.24),
    0 ${headerBoxShadowOffsetY}rem ${headerBoxShadowBlurRadius3Pixel}rem 0
      rgba(0, 0, 0, 0.12);
`;

const HeaderContainer = styled.div`
  flex: 1;
  margin-left: 1rem;
`;

const NavigationAndMainContent = styled.div`
  display: flex;
  justify-content: space-between;
  position: fixed;
  top: ${navigationAndMainContentTop}rem;
  bottom: 0;
  left: 0;
  right: 0;
`;

const scrollbarStyle = css`
  ::-webkit-scrollbar {
    width: ${scrollBarWidth}rem;
  }

  ::-webkit-scrollbar-thumb {
    display: none;
    border-radius: 0.1875rem;
    background-color: rgba(153, 153, 153, 0.8);
  }

  :hover {
    ::-webkit-scrollbar-thumb {
      display: initial;
    }
  }
`;

const navContainerStyles = css`
  flex-shrink: 0;
  position: sticky;
  top: 0;
  overflow-y: scroll;
  ${scrollbarStyle};
`;

const LeftNavContainer = styled.div`
  ${navContainerStyles};
  width: ${navContainerWidth}rem;
  background-color: #f2f2f2;
  border-right: ${navContainerBorderRight}rem solid #e6e6e6;
`;

const NavigationMobile = styled.div`
  display: flex;
  height: 100%;
  position: fixed;
  left: -${navContainerWidth}rem;
  top: 0;
  background-color: #f2f2f2;
  transition: 0.5s;
`;

const NavigationMobileScrollContainer = styled.div`
  width: ${navContainerWidth}rem;
  flex: 1;
  overflow-y: scroll;
  ${scrollbarStyle};
`;

const Content = styled.div`
  flex-grow: 1;
`;

class NavigationDrawer extends React.Component {
  constructor(props) {
    super(props);
  }

  componentDidMount() {
    const content = document.querySelector('#content');
    content.onclick = () => {
      const { openLeftNav } = this.props;

      if (openLeftNav) {
        content.classList.remove('show-transparent');
        document
          .querySelector('#navigation-mobile')
          .classList.remove('show-navigation');
        this.props.onClose();
      }
    };
  }

  componentDidUpdate(prevProps) {
    const { openLeftNav } = this.props;

    if (openLeftNav !== prevProps.openLeftNav) {
      if (openLeftNav) {
        document.querySelector('#content').classList.add('show-transparent');
        document
          .querySelector('#navigation-mobile')
          .classList.add('show-navigation');
      }
    }
  }

  render() {
    const { header, leftNav, children } = this.props;
    return (
      <Container>
        <div id="content">
          <NavigationAndMainContent>
            <LeftNavContainer id="left-nav">{leftNav}</LeftNavContainer>
            <Content>{children}</Content>
          </NavigationAndMainContent>

          <NavButtonHeaderContainer>
            <LeftNavButtonContainer id="navigation-button-for-mobile">
              <LeftNavButton onClick={this.onLeftNavButtonClick} />
            </LeftNavButtonContainer>

            <HeaderContainer id="header-container">{header}</HeaderContainer>
          </NavButtonHeaderContainer>
        </div>
        <NavigationMobile id="navigation-mobile">
          <NavigationMobileScrollContainer>
            {leftNav}
          </NavigationMobileScrollContainer>
        </NavigationMobile>
      </Container>
    );
  }

  onLeftNavButtonClick = () => {
    const { toggleOpenLeftNav } = this.props;

    toggleOpenLeftNav();
  };
}

NavigationDrawer.propTypes = {
  header: PropTypes.node.isRequired,
  leftNav: PropTypes.node.isRequired,
  openLeftNav: PropTypes.bool,
  onClose: PropTypes.func,
  toggleOpenLeftNav: PropTypes.func,
  children: PropTypes.node.isRequired
};

const mapDispatchToProps = { toggleOpenLeftNav };

export default connect(
  undefined,
  mapDispatchToProps
)(NavigationDrawer);
